﻿using System;
using System.Collections.Generic;
using System.Web;

using Microsoft.Practices.Unity;

namespace Palmmedia.Common.Ioc
{
    /// <summary>
    /// <see cref="LifetimeManager"/> that limits the lifetime of an object to the duration of a webrequest.
    /// With help of the HTTP module <see cref="UnityPerWebRequestLifetimeModule"/> objects are disposed at the end of an request.
    /// </summary>
    public class UnityPerWebRequestLifetimeManager : LifetimeManager
    {
        /// <summary>
        /// The HTTP context.
        /// </summary>
        private HttpContextBase httpContext;

        /// <summary>
        /// Initializes a new instance of the <see cref="UnityPerWebRequestLifetimeManager"/> class.
        /// This constructor does only exist for UnitTesting.
        /// </summary>
        /// <param name="httpContext">The HTTP context.</param>
        public UnityPerWebRequestLifetimeManager(HttpContextBase httpContext)
        {
            this.httpContext = httpContext;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UnityPerWebRequestLifetimeManager"/> class.
        /// </summary>
        public UnityPerWebRequestLifetimeManager()
            : this(new HttpContextWrapper(HttpContext.Current))
        {
        }

        /// <summary>
        /// Gets the backing store.
        /// The backing store manages the instances of the <see cref="UnityPerWebRequestLifetimeManager">UnityPerWebRequestLifetimeManagers</see>.
        /// </summary>
        /// <value>The backing store.</value>
        private IDictionary<UnityPerWebRequestLifetimeManager, object> BackingStore
        {
            get
            {
                this.httpContext = (HttpContext.Current != null) ? new HttpContextWrapper(HttpContext.Current) : this.httpContext;
                return UnityPerWebRequestLifetimeModule.GetLifetimeManagerInstances(this.httpContext);
            }
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        /// <value>The value.</value>
        private object Value
        {
            get
            {
                var backingStore = this.BackingStore;

                return backingStore.ContainsKey(this) ? backingStore[this] : null;
            }

            set
            {
                var backingStore = this.BackingStore;

                if (backingStore.ContainsKey(this))
                {
                    object oldValue = backingStore[this];

                    if (!ReferenceEquals(value, oldValue))
                    {
                        IDisposable disposable = oldValue as IDisposable;

                        if (disposable != null)
                        {
                            disposable.Dispose();
                        }

                        if (value == null)
                        {
                            backingStore.Remove(this);
                        }
                        else
                        {
                            backingStore[this] = value;
                        }
                    }
                }
                else
                {
                    if (value != null)
                    {
                        backingStore.Add(this, value);
                    }
                }
            }
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <returns>The value.</returns>
        public override object GetValue()
        {
            return this.Value;
        }

        /// <summary>
        /// Sets the value.
        /// </summary>
        /// <param name="newValue">The new value.</param>
        public override void SetValue(object newValue)
        {
            this.Value = newValue;
        }

        /// <summary>
        /// Removes the value.
        /// </summary>
        public override void RemoveValue()
        {
            this.Value = null;
        }
    }
}