﻿using System;
using System.Collections.Generic;
using System.Web;

namespace Palmmedia.Common.Ioc
{
    /// <summary>
    /// <see cref="IHttpModule"/> that manages all instances of <see cref="UnityPerWebRequestLifetimeManager"/>.
    /// At the end of an request the objects are disposed.
    /// </summary>
    public class UnityPerWebRequestLifetimeModule : IHttpModule
    {
        /// <summary>
        /// Identifier to save and retrieve object in HTTP context.
        /// </summary>
        private static readonly object Key = new object();

        /// <summary>
        /// The HTTP context.
        /// </summary>
        private HttpContextBase httpContext;

        /// <summary>
        /// Initializes a new instance of the <see cref="UnityPerWebRequestLifetimeModule"/> class.
        /// his constructor does only exist for UnitTesting.
        /// </summary>
        /// <param name="httpContext">The HTTP context.</param>
        public UnityPerWebRequestLifetimeModule(HttpContextBase httpContext)
        {
            this.httpContext = httpContext;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UnityPerWebRequestLifetimeModule"/> class.
        /// </summary>
        public UnityPerWebRequestLifetimeModule()
        {
        }

        /// <summary>
        /// Gets the dictionary with <see cref="UnityPerWebRequestLifetimeManager" /> as key and the corresponding object as value.
        /// </summary>
        /// <value>Dictionary with <see cref="UnityPerWebRequestLifetimeManager" /> as key and the corresponding object as value..</value>
        private IDictionary<UnityPerWebRequestLifetimeManager, object> Instances
        {
            get
            {
                this.httpContext = (HttpContext.Current != null) ? new HttpContextWrapper(HttpContext.Current) : this.httpContext;

                return (this.httpContext == null) ? null : GetLifetimeManagerInstances(this.httpContext);
            }
        }

        /// <summary>
        /// Initializes a module and prepares it to handle requests.
        /// </summary>
        /// <param name="context">An <see cref="T:System.Web.HttpApplication"/> that provides access to the methods, properties, and events common to all application objects within an ASP.NET application</param>
        void IHttpModule.Init(HttpApplication context)
        {
            context.EndRequest += (sender, e) => this.RemoveAllInstances();
        }

        /// <summary>
        /// Disposes of the resources (other than memory) used by the module that implements <see cref="T:System.Web.IHttpModule"/>.
        /// </summary>
        void IHttpModule.Dispose()
        {
        }

        /// <summary>
        /// Gets the dictionary with <see cref="UnityPerWebRequestLifetimeManager" /> as key and the corresponding object as value from the HTTP context.
        /// </summary>
        /// <param name="httpContext">The HTTP context.</param>
        /// <returns>The dictionary with <see cref="UnityPerWebRequestLifetimeManager" /> as key and the corresponding object as value.</returns>
        internal static IDictionary<UnityPerWebRequestLifetimeManager, object> GetLifetimeManagerInstances(HttpContextBase httpContext)
        {
            IDictionary<UnityPerWebRequestLifetimeManager, object> instances;

            if (httpContext.Items.Contains(Key))
            {
                instances = (IDictionary<UnityPerWebRequestLifetimeManager, object>) httpContext.Items[Key];
            }
            else
            {
                lock (httpContext.Items)
                {
                    if (httpContext.Items.Contains(Key))
                    {
                        instances = (IDictionary<UnityPerWebRequestLifetimeManager, object>) httpContext.Items[Key];
                    }
                    else
                    {
                        instances = new Dictionary<UnityPerWebRequestLifetimeManager, object>();
                        httpContext.Items.Add(Key, instances);
                    }
                }
            }

            return instances;
        }

        /// <summary>
        /// Removes all instances.
        /// </summary>
        internal void RemoveAllInstances()
        {
            var instances = this.Instances;

            if (instances != null)
            {
                foreach (var entry in instances)
                {
                    var disposable = entry.Value as IDisposable;

                    if (disposable != null)
                    {
                        disposable.Dispose();
                    }
                }

                instances.Clear();
            }
        }
    }
}