﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Web.UI.WebControls;

namespace Palmmedia.Common.Linq
{
    /// <summary>
    /// Extensions for paging and sorting.
    /// </summary>
    public static class PagingExtensions
    {
        /// <summary>
        /// Extends a query with paging and sorting.
        /// </summary>
        /// <typeparam name="T">The type.</typeparam>
        /// <param name="query">The query.</param>
        /// <param name="paging">The <see cref="Paging"/>.</param>
        /// <returns>The extended query.</returns>
		public static IQueryable<T> SortAndPage<T>(this IQueryable<T> query, Paging paging)
        {
            if (paging == null)
            {
                return query;
            }

            if (!string.IsNullOrEmpty(paging.SortColumn))
            {
                // Sorting is required
                var parameter = Expression.Parameter(typeof(T), "p");

                var command = paging.SortDirection == SortDirection.Descending ? "OrderByDescending" : "OrderBy";
                Expression resultExpression = null;
                
                if (paging.SortColumn.Contains('.'))
                {
                    // Sort by nested property, e.g. Orders.Modified
                    var parts = paging.SortColumn.Split(new char[] { '.' }, StringSplitOptions.RemoveEmptyEntries);

                    var property = typeof(T).GetProperty(parts[0]);
                    var nestedProperty = property.PropertyType.GetProperty(parts[1]);

                    var propertyAccess = Expression.MakeMemberAccess(Expression.MakeMemberAccess(parameter, property), nestedProperty);
                    var orderByExpression = Expression.Lambda(propertyAccess, parameter);

                    resultExpression = Expression.Call(
                        typeof(Queryable), 
                        command, 
                        new Type[] { typeof(T), nestedProperty.PropertyType },
                        query.Expression, 
                        Expression.Quote(orderByExpression));
                }                
                else
                {
                    // Sort by simple property
                    var property = typeof(T).GetProperty(paging.SortColumn);
                    var propertyAccess = Expression.MakeMemberAccess(parameter, property);
                    var orderByExpression = Expression.Lambda(propertyAccess, parameter);

                    resultExpression = Expression.Call(
                        typeof(Queryable), 
                        command, 
                        new Type[] { typeof(T), property.PropertyType },
                        query.Expression, 
                        Expression.Quote(orderByExpression));
                }

                query = query.Provider.CreateQuery<T>(resultExpression);
            }

            return query.Skip(paging.PageIndex * paging.PageSize).Take(paging.PageSize);
        }
    }
}