﻿using System;
using System.Collections.Generic;
using System.Data.Objects;
using System.Linq;
using MVCBlog.Core.Repository;

namespace MVCBlog.Core.Database
{
	/// <summary>
	/// Implementation of the <see cref="MVCBlog.Core.Repository.IGenericRepository{T}"/> using the Entity Framework.
	/// </summary>
	/// <typeparam name="T">The <see cref="System.Data.Objects.DataClasses.EntityObject"/>.</typeparam>
	public class EFRepository<T> : IGenericRepository<T> where T : System.Data.Objects.DataClasses.EntityObject
    {
        /// <summary>
        /// The <see cref="ObjectContext"/> to use.
        /// </summary>
        private ObjectContext context;		
		
        /// <summary>
        /// Gets the <see cref="System.Data.Objects.ObjectContext"/> to use.
        /// </summary>
        private ObjectContext Context
        {
            get
            {
                if (this.context == null)
                {
                    this.context = GetCurrentUnitOfWork<EFUnitOfWork>().Context;
                }

                return this.context;
            }
        }		

        /// <summary>
		/// Gets the <see cref="System.Data.Objects.ObjectQuery{T}"/> to use.
        /// </summary>
        private ObjectQuery<T> ObjectQuery
        {
            get
            {
				string entitySetName = GetEntitySetName(typeof(T));				
				return this.Context.CreateQuery<T>(entitySetName);
            }
        }

        /// <summary>
        /// Gets an <see cref="IQueryable{T}"/> to perform further operations.
        /// </summary>
        /// <returns>An <see cref="IQueryable{T}"/>.</returns>
        public IQueryable<T> GetQuery()
        {
            return this.ObjectQuery;
        }

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements.
        /// </summary>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements.</returns>
        public IEnumerable<T> GetAll()
        {
            return this.GetQuery().ToList();
        }

        /// <summary>
        /// Gets an <see cref="IEnumerable{T}"/> of all elements matching the given condition.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>An <see cref="IEnumerable{T}"/> of all elements matching the given condition.</returns>
        public IEnumerable<T> Find(Func<T, bool> where)
        {
            return this.ObjectQuery.Where<T>(where);
        }

        /// <summary>
        /// Gets a single instance matching the given condition.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>A single instance matching the given condition.</returns>
        public T Single(Func<T, bool> where)
        {
            return this.ObjectQuery.Single<T>(where);
        }

        /// <summary>
        /// Gets a single instance matching the given condition or null if condition does not match.
        /// </summary>
        /// <param name="where">The condition.</param>
        /// <returns>A single instance matching the given condition or <c>null</c> if condition does not match.</returns>
        public T SingleOrDefault(Func<T, bool> where)
        {
            return this.ObjectQuery.SingleOrDefault<T>(where);
        }

        /// <summary>
        /// Deletes the given entity.
        /// </summary>
        /// <param name="entity">The entity to delete.</param>
        public void Delete(T entity)
        {
			this.Context.DeleteObject(entity);
        }

        /// <summary>
        /// Adds the given entity.
        /// </summary>
        /// <param name="entity">The entity to add.</param>
        public void Add(T entity)
        {			
			this.Context.AddObject(GetEntitySetName(entity.GetType()), entity);
        }

        /// <summary>
        /// Attaches the the given entity to the current context.
        /// </summary>
        /// <param name="entity">The entity to attach.</param>
        public void Attach(T entity)
        {
            ObjectStateEntry objectStateEntry = null;

            if (!this.Context.ObjectStateManager.TryGetObjectStateEntry(entity, out objectStateEntry))
            {
                this.Context.Attach(entity);
            }
        }

        /// <summary>
        /// Gets the current <see cref="IUnitOfWork"/>.
        /// </summary>
        /// <typeparam name="TUnitOfWork">The Class implementing <see cref="IUnitOfWork"/>.</typeparam>
        /// <returns>The current <see cref="IUnitOfWork"/>.</returns>
        private static TUnitOfWork GetCurrentUnitOfWork<TUnitOfWork>() where TUnitOfWork : class, IUnitOfWork
        {
            return (TUnitOfWork)UnitOfWork.Current;
        }

		/// <summary>
		/// Gets the name of the entity set.
		/// </summary>
		/// <param name="type">The type of the entity.</param>
		/// <returns>The name of the entity set.</returns>
		private static string GetEntitySetName(Type type)
		{
			return type.Name + "Set";
		}
    }
}
