﻿using System;
using System.Collections.Generic;
using System.Linq;
using MVCBlog.Core.Repository;
using Palmmedia.Common.Ioc;
using Palmmedia.Common.Linq;

namespace MVCBlog.Core.Service
{
    /// <summary>
	/// Base class for all services.
    /// </summary>
	/// <typeparam name="T">Class that derives from <see cref="System.Data.Objects.DataClasses.EntityObject"/>.</typeparam>
	public abstract class GenericEntityService<T> : IGenericEntityService<T> where T : System.Data.Objects.DataClasses.EntityObject
    {
		/// <summary>
		/// Initializes a new instance of the <see cref="GenericEntityService{T}"/> class.
		/// </summary>
        protected GenericEntityService()
        {
            this.Repository = IocHelper.Resolve<IGenericRepository<T>>();
        }

        /// <summary>
		/// Gets or sets the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        protected IGenericRepository<T> Repository { get; set; }

        /// <summary>
		/// Adds the given element to the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to add.</param>
        public virtual void Add(T entity)
        {
            this.Add(entity, true);
        }

        /// <summary>
		/// Adds the given element to the <see cref="IGenericRepository{T}"/>.
        /// </summary>
		/// <param name="entity">The element to add.</param>
        /// <param name="saveChanges">Determines wheather changes should be saved.</param>
        internal void Add(T entity, bool saveChanges)
        {
			if (entity == null)
			{
				throw new ArgumentNullException("entity");
			}

            this.Repository.Add(entity);
            if (saveChanges)
            {
				UnitOfWork.Commit();
            }
        }

        /// <summary>
		/// Deletes the given element from the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        /// <param name="entity">Das zu löschende Element.</param>
        public virtual void Delete(T entity)
        {
            this.Delete(entity, true);
        }

        /// <summary>
		/// Deletes the given element from the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        /// <param name="entity">The element to delete.</param>
		/// <param name="saveChanges">Determines wheather changes should be saved.</param>
        internal void Delete(T entity, bool saveChanges)
        {
			if (entity == null)
			{
				throw new ArgumentNullException("entity");
			}

            this.Repository.Delete(entity);
            if (saveChanges)
            {
				UnitOfWork.Commit();
            }
        }

        /// <summary>
		/// Updates the given element in the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        /// <param name="entity">Das zu ändernde Element.</param>
        public virtual void Update(T entity)
        {
            this.Update(entity, true);
        }

        /// <summary>
		/// Updates the given element in the <see cref="IGenericRepository{T}"/>.
        /// </summary>
		/// <param name="entity">The element to update.</param>
		/// <param name="saveChanges">Determines wheather changes should be saved.</param>
        internal void Update(T entity, bool saveChanges)
        {
			if (entity == null)
			{
				throw new ArgumentNullException("entity");
			}

            this.Repository.Attach(entity);

            if (saveChanges)
            {
				UnitOfWork.Commit();
            }
        }

        /// <summary>
		/// Returns an <see cref="IEnumerable{T}"/> from the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        /// <returns>
		/// The <see cref="IEnumerable{T}"/> of all elements.
        /// </returns>
        public virtual IEnumerable<T> GetAll()
        {
            return this.Repository.GetAll();
        }

        /// <summary>
		/// Returns an <see cref="IEnumerable{T}"/> from the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        /// <param name="paging">The <see cref="Paging"/>.</param>
        /// <param name="totalNumberOfElements">Total number of elements.</param>
        /// <returns>
		/// The <see cref="IEnumerable{T}"/> of all elements.
        /// </returns>
        public virtual IEnumerable<T> GetAll(Paging paging, out int totalNumberOfElements)
        {
			var query = this.Repository.GetQuery();
            totalNumberOfElements = query.Count();
            return query.SortAndPage(paging);
        }
    }
}
