﻿using System;
using System.Collections.Generic;
using System.Linq;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Repository;
using Palmmedia.Common.Ioc;
using Palmmedia.Common.Linq;

namespace MVCBlog.Core.Service
{
    /// <summary>
	/// Access to <see cref="BlogEntry">BlogEntries</see>.
    /// </summary>
	public class BlogEntryService : GenericEntityService<BlogEntry>, IBlogEntryService
    {
		/// <summary>
		/// Returns the <see cref="BlogEntry"/> with the given Id.
		/// </summary>
		/// <param name="id">The Id.</param>
		/// <returns>
		/// The <see cref="BlogEntry"/> with the given Id.
		/// </returns>
		public BlogEntry GetById(Guid id)
		{
			var entry = this.Repository.SingleOrDefault(e => e.Id == id);
            if (entry != null)
            {
                entry.Tags.Load();
                entry.Files.Load();
                entry.Comments.Load();
            }

            return entry;
		}

        /// <summary>
        /// Returns the <see cref="BlogEntry"/> with the given header.
        /// </summary>
        /// <param name="includeInvisibleEntries">Determines whether invisible <see cref="BlogEntry">BlogEntries</see> should be included.</param>
        /// <param name="header">The header.</param>
        /// <returns>
        /// The <see cref="BlogEntry"/> with the given header.
        /// </returns>
        public BlogEntry GetByHeader(bool includeInvisibleEntries, string header)
        {
            BlogEntry entry = null;

            if (includeInvisibleEntries)
            {
                entry = this.Repository.SingleOrDefault(e => e.HeaderUrl.Equals(header));
            }
            else
            {
                entry = this.Repository.SingleOrDefault(e => e.Visible && e.HeaderUrl.Equals(header));
            }            
            
            if (entry != null)
            {
                entry.Tags.Load();
                entry.Files.Load();
                entry.Comments.Load();
            }

            return entry;
        }

        /// <summary>
        /// Returns all <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <param name="includeInvisibleEntries">Determines whether invisible <see cref="BlogEntry">BlogEntries</see> should be included.</param>
        /// <param name="tag">The text of the tag.</param>
        /// <param name="search">The search.</param>
        /// <param name="paging">The <see cref="Paging"/>.</param>
        /// <param name="totalNumberOfElements">Total number of elements.</param>
        /// <returns>
        /// All <see cref="BlogEntry">BlogEntries</see>.
        /// </returns>
        public IEnumerable<BlogEntry> GetAll(bool includeInvisibleEntries, string tag, string search, Paging paging, out int totalNumberOfElements)
        {
            var query = this.Repository.GetQuery();

            if (!includeInvisibleEntries)
            {
                query = query.Where(e => e.Visible);
            }

            if (!string.IsNullOrEmpty(tag))
            {
                query = query.Where(e => e.Tags.Count(t => t.Name.Equals(tag, StringComparison.OrdinalIgnoreCase)) > 0);              
            }

            if (!string.IsNullOrEmpty(search))
            {
                foreach (var item in search.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries))
                {
                    query = query.Where(e => e.Header.Contains(item));
                }
            }
            
            totalNumberOfElements = query.Count();
            var entries = query.SortAndPage(paging);

            foreach (var entry in entries)
            {
                entry.Comments.Load();
            }

            return entries;
        }

        /// <summary>
        /// Returns an <see cref="IEnumerable{T}"/> from the <see cref="IGenericRepository{T}"/>.
        /// </summary>
        /// <returns>
        /// The <see cref="IEnumerable{T}"/> of all elements.
        /// </returns>
        public override IEnumerable<BlogEntry> GetAll()
        {
            var entries = this.Repository.GetQuery();

            entries = entries.Where(e => e.Visible).OrderByDescending(e => e.Created);

            foreach (var entry in entries)
            {
                entry.Tags.Load();
            }

            return entries;
        }

        /// <summary>
        /// Returns related <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <param name="entry">The <see cref="BlogEntry"/>.</param>
        /// <param name="numberOfEntries">Maximum number of <see cref="BlogEntry">BlogEntries</see>.</param>
        /// <returns>
        /// The related <see cref="BlogEntry">BlogEntries</see>.
        /// </returns>
        public IEnumerable<BlogEntry> GetRelatedBlogEntries(BlogEntry entry, int numberOfEntries)
        {
            if (entry == null)
            {
                throw new ArgumentNullException("entry");
            }

            /* Search for BlogEntries with the most corresponding Tags.
             * Since EF does not support the Contains() method, a workaround is required.
             * Only the first 6 tags are used.
             * The upcoming EF 4.0 will support the Contains() method.
             * 
             * This approach does not work here:
             * http://stackoverflow.com/questions/374267/contains-workaround-using-linq-to-entities
             * Since it is designed to work on ObjectQuery<TEntity>, but here we want to create a query on a nested
             * property which is of type EntityCollection<TEntity>.
             */
            var tagIds = entry.Tags.Select(t => t.Id).ToArray();

            var id1 = tagIds.Length > 0 ? tagIds[0] : Guid.Empty;
            var id2 = tagIds.Length > 1 ? tagIds[1] : Guid.Empty;
            var id3 = tagIds.Length > 2 ? tagIds[2] : Guid.Empty;
            var id4 = tagIds.Length > 3 ? tagIds[3] : Guid.Empty;
            var id5 = tagIds.Length > 4 ? tagIds[4] : Guid.Empty;
            var id6 = tagIds.Length > 5 ? tagIds[5] : Guid.Empty;

            var query = this.Repository.GetQuery().Where(e => e.Visible && e.Id != entry.Id && e.Tags.Count(t => t.Id == id1 || t.Id == id2 || t.Id == id3 || t.Id == id4 || t.Id == id5 || t.Id == id6) > 0);
            var orderedQuery = query.OrderByDescending(e => e.Tags.Count(t => t.Id == id1 || t.Id == id2 || t.Id == id3 || t.Id == id4 || t.Id == id5 || t.Id == id6));
            orderedQuery = orderedQuery.ThenByDescending(e => e.Created);
            return orderedQuery.Take(numberOfEntries);
        }

		/// <summary>
		/// Adds the given <see cref="BlogEntry"/>.
		/// </summary>
		/// <param name="entry">The <see cref="BlogEntry"/> to add.</param>
		/// <param name="tags">The tags.</param>
		public void Add(BlogEntry entry, IEnumerable<string> tags)
		{
			if (entry == null)
			{
				throw new ArgumentNullException("entry");
			}

			if (tags == null)
			{
				throw new ArgumentNullException("tags");
			}

			AddTags(entry, tags);
			base.Add(entry);
		}

		/// <summary>
		/// Updates the given <see cref="BlogEntry"/>.
		/// </summary>
		/// <param name="entry">The <see cref="BlogEntry"/> to update.</param>
		/// <param name="tags">The tags.</param>
		public void Update(BlogEntry entry, IEnumerable<string> tags)
		{
			if (entry == null)
			{
				throw new ArgumentNullException("entry");
			}

			if (tags == null)
			{
				throw new ArgumentNullException("tags");
			}

			AddTags(entry, tags);
			base.Update(entry);
		}

		/// <summary>
		/// Deletes the given element from the <see cref="IGenericRepository{T}"/>.
		/// </summary>
		/// <param name="entity">Das zu löschende Element.</param>
		public override void Delete(BlogEntry entity)
		{
			if (entity == null)
			{
				throw new ArgumentNullException("entity");
			}

			foreach (var blogEntryFile in entity.Files)
			{
				blogEntryFile.DeleteData();
			}

			base.Delete(entity);
		}

		/// <summary>
		/// Adds the file to the <see cref="BlogEntry"/> with the given Id.
		/// </summary>
		/// <param name="id">The id.</param>
		/// <param name="fileName">The filename.</param>
		/// <param name="file">The file.</param>
		public void AddFile(Guid id, string fileName, byte[] file)
		{
			if (fileName == null)
			{
				throw new ArgumentNullException("fileName");
			}

			if (file == null)
			{
				throw new ArgumentNullException("file");
			}

			var entry = this.Repository.SingleOrDefault(e => e.Id == id);

			if (entry != null)
			{
				int indexOfLastDot = fileName.LastIndexOf('.');
				string name = fileName.Substring(0, indexOfLastDot);
				string extension = fileName.Substring(indexOfLastDot + 1, fileName.Length - indexOfLastDot - 1);

				var blogEntryFile = new BlogEntryFile() { BlogEntry = entry, Name = name, Extension = extension };
				blogEntryFile.Data = file;

				entry.Files.Add(blogEntryFile);
				UnitOfWork.Commit();
			}
		}

		/// <summary>
		/// Adds the tags to the given <see cref="BlogEntry"/>.
		/// </summary>
		/// <param name="entry">The entry.</param>
		/// <param name="tags">The tags.</param>
		private static void AddTags(BlogEntry entry, IEnumerable<string> tags)
		{
			entry.Tags.Clear();

            var tagService = IocHelper.Resolve<ITagService>();

			foreach (var text in tags)
			{
                if (entry.Tags.SingleOrDefault(t => t.Name.Equals(text, StringComparison.OrdinalIgnoreCase)) != null)
                {
                    continue;
                }

				var tag = tagService.GetByText(text);
				if (tag == null)
				{
					tag = new Tag() { Name = text };
				}

				tag.BlogEntries.Add(entry);
				entry.Tags.Add(tag);
			}
		}
    }
}
