﻿using System.Configuration;
using System.Text.RegularExpressions;
using CookComputing.XmlRpc;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Service;
using Palmmedia.Common.Ioc;
using Palmmedia.Common.Net.PingBack;

namespace MVCBlog.Website
{
    /// <summary>
    /// <see cref="XmlRpcService"/> handling pingback requests.
    /// </summary>
    public class PingbackHandler : XmlRpcService
    {
        /// <summary>
        /// The <see cref="BlogEntry"/> a pingback is is added to. 
        /// </summary>
        private BlogEntry blogEntry;

        #region Properties
        /// <summary>
        /// The blog entry service.
        /// </summary>
        private IBlogEntryService blogEntryService;

        /// <summary>
        /// Gets or sets the blog entry service.
        /// </summary>
        public IBlogEntryService BlogEntryService
        {
            get
            {
                if (this.blogEntryService == null)
                {
                    this.blogEntryService = IocHelper.Resolve<IBlogEntryService>();
                }

                return this.blogEntryService;
            }

            set
            {
                this.blogEntryService = value;
            }
        }

        /// <summary>
        /// The blog entry comment service.
        /// </summary>
        private IBlogEntryCommentService blogEntryCommentService;

        /// <summary>
        /// Gets or sets the blog entry comment service.
        /// </summary>
        public IBlogEntryCommentService BlogEntryCommentService
        {
            get
            {
                if (this.blogEntryCommentService == null)
                {
                    this.blogEntryCommentService = IocHelper.Resolve<IBlogEntryCommentService>();
                }

                return this.blogEntryCommentService;
            }

            set
            {
                this.blogEntryCommentService = value;
            }
        }
        #endregion

        /// <summary>
        /// Receives a pingback request.
        /// </summary>
        /// <param name="sourceUri">The source URI.</param>
        /// <param name="targetUri">The target URI.</param>
        /// <returns>The success message.</returns>
        [XmlRpcMethod("pingback.ping")]
        public string Pingback(string sourceUri, string targetUri)
        {
            PingBackService.ValidatePingbackRequest(
                sourceUri, 
                targetUri,
                this.IsTargetPingbackEnabled,
                this.DoesPingbackAlreadyExist,
                this.RegisterPingback);

            return "Your ping request has been received successfully.";
        }

        /// <summary>
        /// Checks whether the target URI can receive pingbacks.
        /// </summary>
        /// <param name="targetUri">The target URI.</param>
        /// <returns><c>true</c> if target URI points to a <see cref="BlogEntry"/>, otherwise <c>false</c>.</returns>
        private bool IsTargetPingbackEnabled(string targetUri)
        {           
            var match = Regex.Match(targetUri, ConfigurationManager.AppSettings["PingbackPattern"], RegexOptions.IgnoreCase);

            if (match.Success)
            {
                this.blogEntry = this.BlogEntryService.GetByHeader(false, match.Groups[1].Value);
                return this.blogEntry != null;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Checks if a pingback from the source URI has already been registered.
        /// </summary>
        /// <param name="sourceUri">The source URI.</param>
        /// <returns><c>true</c> if pingback does already exist <see cref="BlogEntry"/>, otherwise <c>false</c>.</returns>
        private bool DoesPingbackAlreadyExist(string sourceUri)
        {
            return this.BlogEntryCommentService.DoesPingbackExist(this.blogEntry.Id, sourceUri);
        }

        /// <summary>
        /// Registers the pingback.
        /// </summary>
        /// <param name="sourceUri">The source URI.</param>
        private void RegisterPingback(string sourceUri)
        {
            var blogEntryComment = new BlogEntryComment();

            blogEntryComment.Comment = sourceUri;
            blogEntryComment.Pingback = true;
            blogEntryComment.BlogEntry = this.blogEntry;
            this.blogEntry.Comments.Add(blogEntryComment);

            this.BlogEntryCommentService.Add(blogEntryComment);
        }        
    }
}