using System.Web.Mvc;
using System.Web.Security;
using MVCBlog.Core.Service;
using Palmmedia.Common.Ioc;
using Palmmedia.Common.Net.Mvc.Localization;

namespace MVCBlog.Website.Controllers
{
    /// <summary>
    /// Controller for the administration tasks.
    /// </summary>
    [Palmmedia.Common.Net.Mvc.HandleError]
    public partial class AdminController : PingbackControllerBase
    {
        #region Properties
        /// <summary>
        /// The blog entry file service.
        /// </summary>
        private IBlogEntryFileService blogEntryFileService;

        /// <summary>
        /// Gets or sets the blog entry file service.
        /// </summary>
        public IBlogEntryFileService BlogEntryFileService
        {
            get
            {
                if (this.blogEntryFileService == null)
                {
                    this.blogEntryFileService = IocHelper.Resolve<IBlogEntryFileService>();
                }

                return this.blogEntryFileService;
            }

            set
            {
                this.blogEntryFileService = value;
            }
        }
        #endregion

        /// <summary>
        /// Shows an overview.
        /// </summary>
        /// <returns>View showing an overview.</returns>
        [Authorize()]
		public virtual ActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Shows all downloads.
        /// </summary>
        /// <returns>View showing all downloads.</returns>
        [Authorize()]
		public virtual ActionResult Downloads()
		{
            var blogEntryFiles = this.BlogEntryFileService.GetAll();

            return View(blogEntryFiles);
		}

        /// <summary>
        /// Shows the login form.
        /// </summary>
        /// <returns>View showing the login form.</returns>
		public virtual ActionResult Login()
		{
			return View();
		}

        /// <summary>
        /// Validates the login and redirects to the return URL.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <param name="rememberMe">If set to <c>true</c> a cookie is stored.</param>
        /// <param name="returnUrl">The return URL.</param>
        /// <returns>Redirect to the return URL.</returns>
        [Palmmedia.Common.Net.Mvc.ReferrerAuthorize()]
        [ValidateAntiForgeryToken()]
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Login(string userName, string password, bool rememberMe, string returnUrl)
        {
            if (!this.ValidateLogin(userName, password))
            {
                return View();
            }

            if (!FormsAuthentication.Authenticate(userName, password))
            {
                ModelState.AddModelError("login", this.Resource("AdminController, Login"));
                return View();
            }

            FormsAuthentication.SetAuthCookie(userName, rememberMe);

            if (!string.IsNullOrEmpty(returnUrl))
            {
                return Redirect(returnUrl);
            }
            else
            {
                return this.RedirectToAction(MVC.Admin.Index());
            }
        }

        /// <summary>
        /// Shows the logout form.
        /// </summary>
        /// <returns>View showing the logout form.</returns>
		public virtual ActionResult Logout()
		{
			return View();
		}

        /// <summary>
        /// Perfoms the the logout.
        /// </summary>
        /// <param name="formValues">The form values.</param>
        /// <returns>Redirect to start page.</returns>
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Logout(FormCollection formValues)
        {
            FormsAuthentication.SignOut();

            return this.RedirectToAction(MVC.Blog.Index());
        }

        /// <summary>
        /// Validates a login.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <returns>True if login is valid.</returns>
        private bool ValidateLogin(string userName, string password)
        {
            if (string.IsNullOrEmpty(userName))
            {
                ModelState.AddModelError("userName", this.Resource("AdminController, Username"));
            }

            if (string.IsNullOrEmpty(password))
            {
                ModelState.AddModelError("password", this.Resource("AdminController, Password"));
            }

            return ModelState.IsValid;
        }
    }
}
