using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.ServiceModel.Syndication;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using System.Web.UI.WebControls;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Service;
using MVCBlog.Website.Models;
using MvcSiteMap.Core.Filters;
using Palmmedia.Common;
using Palmmedia.Common.Ioc;
using Palmmedia.Common.Linq;
using Palmmedia.Common.Net.Mvc.Localization;

namespace MVCBlog.Website.Controllers
{
    /// <summary>
    /// Controller for the blog.
    /// </summary>
    [Palmmedia.Common.Net.Mvc.HandleError]
    public partial class BlogController : PingbackControllerBase
    {
        /// <summary>
        /// Number of <see cref="BlogEntry">BlogEntries</see> per page.
        /// </summary>
        private const int ENTRIESPERPAGE = 8;

        #region Properties
        /// <summary>
        /// The blog entry service.
        /// </summary>
        private IBlogEntryService blogEntryService;

        /// <summary>
        /// Gets or sets the blog entry service.
        /// </summary>
        public IBlogEntryService BlogEntryService
        {
            get
            {
                if (this.blogEntryService == null)
                {
                    this.blogEntryService = IocHelper.Resolve<IBlogEntryService>();
                }

                return this.blogEntryService;
            }

            set
            {
                this.blogEntryService = value;
            }
        }

        /// <summary>
        /// The image service.
        /// </summary>
        private IImageService imageService;

        /// <summary>
        /// Gets or sets the image service.
        /// </summary>
        public IImageService ImageService
        {
            get
            {
                if (this.imageService == null)
                {
                    this.imageService = IocHelper.Resolve<IImageService>();
                }

                return this.imageService;
            }

            set
            {
                this.imageService = value;
            }
        }

        /// <summary>
        /// The tag service.
        /// </summary>
        private ITagService tagService;

        /// <summary>
        /// Gets or sets the tag service.
        /// </summary>
        public ITagService TagService
        {
            get
            {
                if (this.tagService == null)
                {
                    this.tagService = IocHelper.Resolve<ITagService>();
                }

                return this.tagService;
            }

            set
            {
                this.tagService = value;
            }
        }

        /// <summary>
        /// The blog entry comment service.
        /// </summary>
        private IBlogEntryCommentService blogEntryCommentService;

        /// <summary>
        /// Gets or sets the blog entry comment service.
        /// </summary>
        public IBlogEntryCommentService BlogEntryCommentService
        {
            get
            {
                if (this.blogEntryCommentService == null)
                {
                    this.blogEntryCommentService = IocHelper.Resolve<IBlogEntryCommentService>();
                }

                return this.blogEntryCommentService;
            }

            set
            {
                this.blogEntryCommentService = value;
            }
        }

        /// <summary>
        /// The blog entry file service.
        /// </summary>
        private IBlogEntryFileService blogEntryFileService;

        /// <summary>
        /// Gets or sets the blog entry file service.
        /// </summary>
        public IBlogEntryFileService BlogEntryFileService
        {
            get
            {
                if (this.blogEntryFileService == null)
                {
                    this.blogEntryFileService = IocHelper.Resolve<IBlogEntryFileService>();
                }

                return this.blogEntryFileService;
            }

            set
            {
                this.blogEntryFileService = value;
            }
        }
        #endregion

        /// <summary>
        /// Shows all <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <param name="tag">The tag.</param>
        /// <param name="search">The search.</param>
        /// <param name="page">Number of the page.</param>
        /// <returns>A view showing all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [ValidateInput(false)]
		public virtual ActionResult Index(string tag, string search, int? page)
        {
            int totalNumberOfEntries = 0;
            int pageIndex = page.HasValue && page.Value > 0 ? page.Value - 1 : 0;
            var entries = this.BlogEntryService.GetAll(
                this.Request.IsAuthenticated, 
                tag, 
                search, 
                new Paging(pageIndex, ENTRIESPERPAGE, "Created", SortDirection.Descending), 
                out totalNumberOfEntries);

            var model = new BlogEntries();
            model.Entries = entries.Select(e => new BlogEntryDetail() { BlogEntry = e });
            model.TotalPages = (int)Math.Ceiling((double)totalNumberOfEntries / ENTRIESPERPAGE);
            model.CurrentPage = page;
            model.Tag = tag;
            model.Tags = this.TagService.GetAll();
            model.Search = search;
            return View(model);
        }

        /// <summary>
        /// Shows a single <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [SiteMapTitle("Header")]
        public virtual ActionResult Entry(string id)
        {
            var entry = this.BlogEntryService.GetByHeader(this.Request.IsAuthenticated, id);

            if (entry == null)
            {
                return this.RedirectToAction(MVC.Error.NotFound());
            }

            var relatedEntries = this.BlogEntryService.GetRelatedBlogEntries(entry, 3);

            return View(new BlogEntryDetail() 
            { 
                BlogEntry = entry, 
                Header = entry.Header, 
                ShowDetails = true, 
                Tags = this.TagService.GetAll(), 
                RelatedBlogEntries = relatedEntries 
            });
        }

        /// <summary>
        /// Adds a <see cref="BlogEntryComment"/> to a <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="name">The name.</param>
        /// <param name="email">The email.</param>
        /// <param name="homepage">The homepage.</param>
        /// <param name="comment">The comment.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [SiteMapTitle("Header")]
        [ValidateInput(false)]
        [Palmmedia.Common.Net.Mvc.ReferrerAuthorize()]
        [Palmmedia.Common.Net.Mvc.SpamProtection()]
        [ValidateAntiForgeryToken()]
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Entry(string id, string name, string email, string homepage, string comment)
        {
            var entry = this.BlogEntryService.GetByHeader(this.Request.IsAuthenticated, id);

            if (entry == null)
            {
                return this.RedirectToAction(MVC.Error.NotFound());
            }

            if (!this.ValidateComment(name, email, homepage, comment))
            {
                var errorModel = new BlogEntryDetail()
                {
                    BlogEntry = entry,
                    Header = entry.Header,
                    ShowDetails = true
                };

                if (this.Request.IsAjaxRequest())
                {
                    return PartialView(MVC.Blog.Views.CommentsControl, errorModel);
                }
                else
                {
                    errorModel.Tags = this.TagService.GetAll();
                    errorModel.RelatedBlogEntries = this.BlogEntryService.GetRelatedBlogEntries(entry, 3);
                    return View(errorModel);
                }
            }

            var blogEntryComment = new BlogEntryComment();            
            blogEntryComment.Name = name;
            blogEntryComment.Email = email.GetTextOrNullIfEmpty();
            blogEntryComment.Homepage = homepage.GetTextOrNullIfEmpty();
            blogEntryComment.Comment = comment;
            blogEntryComment.AdminPost = this.Request.IsAuthenticated;
            blogEntryComment.BlogEntry = entry;
            entry.Comments.Add(blogEntryComment);

            this.BlogEntryCommentService.Add(blogEntryComment);

            var model = new BlogEntryDetail()
            {
                BlogEntry = entry,
                Header = entry.Header,
                ShowDetails = true,
                HideNewCommentsForm = true
            };

            if (this.Request.IsAjaxRequest())
            {
                return PartialView(MVC.Blog.Views.CommentsControl, model);
            }
            else
            {
                model.Tags = this.TagService.GetAll();
                model.RelatedBlogEntries = this.BlogEntryService.GetRelatedBlogEntries(entry, 3);
                return View(model);
            }        
        }

        /// <summary>
        /// Returns an RSS-Feed of all <see cref="BlogEntry">BlogEntries</see>.
        /// </summary>
        /// <returns>RSS-Feed of all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [OutputCache(CacheProfile = "Long")]
        public virtual ActionResult Feed()
        {
            var entries = this.BlogEntryService.GetAll();

            string baseUrl = this.Request.Url.GetLeftPart(UriPartial.Authority);

            var feed = new SyndicationFeed(
                ConfigurationManager.AppSettings["BlogName"],
                ConfigurationManager.AppSettings["BlogDescription"], 
                new Uri(baseUrl + "/" + MVC.Blog.Name + "/" + MVC.Blog.Actions.Feed));

            feed.BaseUri = new Uri(baseUrl);

            if (entries.Count() > 0)
            {
                DateTime lastCreatedDate = entries.ElementAt(0).Created;

                DateTime? lastModifiedDate = entries.OrderByDescending(e => e.Modified).First().Modified;
                if (lastModifiedDate.HasValue && lastModifiedDate.Value > lastCreatedDate)
                {
                    feed.LastUpdatedTime = lastModifiedDate.Value;
                }
                else
                {
                    feed.LastUpdatedTime = lastCreatedDate;
                }
            }

            var feedItems = new List<SyndicationItem>();

            foreach (var blogEntry in entries)
            {
                var syndicationItem = new SyndicationItem(
                    blogEntry.Header,
                    SyndicationContent.CreateXhtmlContent(blogEntry.Content),
                    new Uri(baseUrl + "/" + MVC.Blog.Name + "/" + blogEntry.Url),
                    blogEntry.Id.ToString(),
                    blogEntry.Modified.HasValue ? blogEntry.Modified.Value : blogEntry.Created);

                foreach (var tag in blogEntry.Tags)
                {
                    syndicationItem.Categories.Add(new SyndicationCategory(tag.Name));
                }

                feedItems.Add(syndicationItem);
            }

            feed.Items = feedItems;

            return new Palmmedia.Common.Net.Mvc.Feed.RssSyndicationFeedActionResult(feed);
        }

        /// <summary>
        /// Shows all <see cref="BlogEntryComment">BlogEntryComments</see>.
        /// </summary>
        /// <returns>A view showing all <see cref="BlogEntryComment">BlogEntryComments</see>.</returns>
        [Authorize()]
		public virtual ActionResult Comments()
		{
            var comments = this.BlogEntryCommentService.GetAll();
			return View(comments);
		}

        /// <summary>
        /// Shows all <see cref="MVCBlog.Core.Entities.Image">Images</see>.
        /// </summary>
        /// <returns>A view showing all <see cref="MVCBlog.Core.Entities.Image">Images</see>.</returns>
        [Authorize()]
        public virtual ActionResult Images()
        {
            var images = this.ImageService.GetAll();
            return View(images);
        }

        /// <summary>
        /// Upload an image
        /// </summary>
        /// <returns>A view showing all <see cref="MVCBlog.Core.Entities.Image">Images</see>.</returns>
        [AcceptVerbs(HttpVerbs.Post)]
        [Authorize()]
        public virtual ActionResult ImageUpload()
        {
            foreach (string file in Request.Files)
            {
                var hpf = Request.Files[file] as HttpPostedFileBase;
                if (hpf.ContentLength == 0)
                {
                    continue;
                }

                var content = new byte[hpf.ContentLength];
                hpf.InputStream.Read(content, 0, hpf.ContentLength);
                
                this.ImageService.Add(hpf.FileName, content);
            }

            return this.RedirectToAction(MVC.Blog.Images());
        }

        /// <summary>
        /// Deletes the <see cref="BlogEntry"/> with the given id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>A view showing all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [Authorize()]
        public virtual ActionResult Delete(Guid id)
        {
            var blogEntry = this.BlogEntryService.GetById(id);

            this.BlogEntryService.Delete(blogEntry);

            return this.RedirectToAction(MVC.Blog.Index());
        }

        /// <summary>
        /// Deletes the <see cref="BlogEntryComment"/> with the given id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>A view showing all <see cref="BlogEntry">BlogEntries</see>.</returns>
        [Authorize()]
        public virtual ActionResult DeleteComment(Guid id)
        {
            var blogEntryComment = this.BlogEntryCommentService.GetById(id);

            this.BlogEntryCommentService.Delete(blogEntryComment);

            return this.RedirectToAction(MVC.Blog.Index());
        }

        /// <summary>
        /// Streams the <see cref="BlogEntryFile"/> with the given id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>The <see cref="BlogEntryFile"/> as Download.</returns>
        public virtual ActionResult Download(Guid id)
        {
            var blogEntryFile = this.BlogEntryFileService.GetById(id);

            if (blogEntryFile == null)
            {
                return this.RedirectToAction(MVC.Error.NotFound());
            }

            blogEntryFile.Counter++;
            this.BlogEntryFileService.Update(blogEntryFile);
            
            var file = new FileContentResult(blogEntryFile.Data, "application/octet-stream");
            file.FileDownloadName = blogEntryFile.Name + "." + blogEntryFile.Extension;
            return file;
        }

        /// <summary>
        /// Validates a new comment.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="email">The email.</param>
        /// <param name="homepage">The homepage.</param>
        /// <param name="comment">The comment.</param>
        /// <returns>True if comment is valid.</returns>
        private bool ValidateComment(string name, string email, string homepage, string comment)
        {
            if (string.IsNullOrEmpty(name))
            {
                ModelState.AddModelError("name", this.Resource("BlogController, Name"));
            }

            if (!string.IsNullOrEmpty(email) && !Regex.IsMatch(email, @"\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*"))
            {
                ModelState.AddModelError("email", this.Resource("BlogController, Email"));
            }

            if (!string.IsNullOrEmpty(homepage) && !Regex.IsMatch(homepage, @"http(s)?://([\w-]+\.)+[\w-]+(/[\w- ./?%&=]*)?"))
            {
                ModelState.AddModelError("homepage", this.Resource("BlogController, Homepage"));
            }

            if (string.IsNullOrEmpty(comment))
            {
                ModelState.AddModelError("comment", this.Resource("BlogController, Comment"));
            }

            return ModelState.IsValid;
        }
    }
}
