using System;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MVCBlog.Core.Entities;
using MVCBlog.Core.Service;
using MVCBlog.Website.Models;
using Palmmedia.Common.Ioc;

namespace MVCBlog.Website.Controllers
{
    /// <summary>
    /// Controller for editing <see cref="BlogEntry">BlogEntries</see>.
    /// </summary>
    [Authorize()]
    [Palmmedia.Common.Net.Mvc.HandleError]
    public partial class BlogEntryEditController : PingbackControllerBase
    {
        #region Properties
        /// <summary>
        /// The blog entry service.
        /// </summary>
        private IBlogEntryService blogEntryService;

        /// <summary>
        /// Gets or sets the blog entry service.
        /// </summary>
        public IBlogEntryService BlogEntryService
        {
            get
            {
                if (this.blogEntryService == null)
                {
                    this.blogEntryService = IocHelper.Resolve<IBlogEntryService>();
                }

                return this.blogEntryService;
            }

            set
            {
                this.blogEntryService = value;
            }
        }

        /// <summary>
        /// The image service.
        /// </summary>
        private IImageService imageService;

        /// <summary>
        /// Gets or sets the image service.
        /// </summary>
        public IImageService ImageService
        {
            get
            {
                if (this.imageService == null)
                {
                    this.imageService = IocHelper.Resolve<IImageService>();
                }

                return this.imageService;
            }

            set
            {
                this.imageService = value;
            }
        }

        /// <summary>
        /// The tag service.
        /// </summary>
        private ITagService tagService;

        /// <summary>
        /// Gets or sets the tag service.
        /// </summary>
        public ITagService TagService
        {
            get
            {
                if (this.tagService == null)
                {
                    this.tagService = IocHelper.Resolve<ITagService>();
                }

                return this.tagService;
            }

            set
            {
                this.tagService = value;
            }
        }

        /// <summary>
        /// The blog entry comment service.
        /// </summary>
        private IBlogEntryCommentService blogEntryCommentService;

        /// <summary>
        /// Gets or sets the blog entry comment service.
        /// </summary>
        public IBlogEntryCommentService BlogEntryCommentService
        {
            get
            {
                if (this.blogEntryCommentService == null)
                {
                    this.blogEntryCommentService = IocHelper.Resolve<IBlogEntryCommentService>();
                }

                return this.blogEntryCommentService;
            }

            set
            {
                this.blogEntryCommentService = value;
            }
        }

        /// <summary>
        /// The blog entry file service.
        /// </summary>
        private IBlogEntryFileService blogEntryFileService;

        /// <summary>
        /// Gets or sets the blog entry file service.
        /// </summary>
        public IBlogEntryFileService BlogEntryFileService
        {
            get
            {
                if (this.blogEntryFileService == null)
                {
                    this.blogEntryFileService = IocHelper.Resolve<IBlogEntryFileService>();
                }

                return this.blogEntryFileService;
            }

            set
            {
                this.blogEntryFileService = value;
            }
        }
        #endregion

        /// <summary>
        /// Show a single <see cref="BlogEntry"/> for editing.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        public virtual ActionResult Index(Guid? id)
        {
            var blogEntry = id.HasValue ? this.BlogEntryService.GetById(id.Value) : new BlogEntry();

            return View(new BlogEntryEdit() { BlogEntry = blogEntry, Tags = this.TagService.GetAll(), Images = this.ImageService.GetAll(), IsUpdate = id.HasValue });
        }

        /// <summary>
        /// Saves the changes applied to the <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <param name="formValues">The form values.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [ValidateInput(false)]
        [Palmmedia.Common.Net.Mvc.ReferrerAuthorize()]
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Index(Guid? id, FormCollection formValues)
        {
            var blogEntry = id.HasValue ? this.BlogEntryService.GetById(id.Value) : new BlogEntry();

            this.UpdateModel<BlogEntry>(blogEntry, null, new string[] { "Header", "Author", "ShortContent", "Content", "Visible" });

            if (!this.ValidatePost(blogEntry.Header, blogEntry.Content))
            {
                return View(new BlogEntryEdit() { BlogEntry = blogEntry, Tags = this.TagService.GetAll(), Images = this.ImageService.GetAll(), IsUpdate = id.HasValue });
            }

            if (string.IsNullOrEmpty(blogEntry.ShortContent))
            {
                blogEntry.ShortContent = null;
            }

            if (string.IsNullOrEmpty(blogEntry.Author))
            {
                blogEntry.Author = null;
            }

            var tags = formValues.AllKeys.Where(k => k.StartsWith("Tag") && !string.IsNullOrEmpty(formValues[k])).Select(k => formValues[k]);

            if (id.HasValue)
            {
                this.BlogEntryService.Update(blogEntry, tags);
            }
            else
            {
                this.BlogEntryService.Add(blogEntry, tags);
            }

            return this.RedirectToAction(MVC.BlogEntryEdit.Index(blogEntry.Id));
        }

        /// <summary>
        /// Performs all pingback requests. All URLs mentioned in the BlogPost are notified.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <returns>A view showing the results of the pingback requests.</returns>
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult PerformPingbacks(Guid id)
        {
            var blogEntry = this.BlogEntryService.GetById(id);
            var pingbackResults = Palmmedia.Common.Net.PingBack.PingBackService.PerformPingBack(
                this.Request.Url.GetLeftPart(UriPartial.Authority) + "/" + MVC.Blog.Name + "/" + blogEntry.Url, 
                blogEntry.Content);

            return View(MVC.BlogEntryEdit.Views.Pingback, pingbackResults);
        }

        /// <summary>
        /// Adds a file to a <see cref="BlogEntry"/>.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult FileUpload(Guid id)
        {
            foreach (string file in Request.Files)
            {
                var hpf = Request.Files[file] as HttpPostedFileBase;
                if (hpf.ContentLength == 0)
                {
                    continue;
                }

                var content = new byte[hpf.ContentLength];
                hpf.InputStream.Read(content, 0, hpf.ContentLength);

                this.BlogEntryService.AddFile(id, hpf.FileName, content);
            }

            return this.RedirectToAction(MVC.BlogEntryEdit.Index(id));
        }

        /// <summary>
        /// Files the delete.
        /// </summary>
        /// <param name="id">The Id of the <see cref="BlogEntry"/>.</param>
        /// <param name="fileId">The file id.</param>
        /// <returns>A view showing a single <see cref="BlogEntry"/>.</returns>
        public virtual ActionResult FileDelete(Guid id, Guid fileId)
        {
            var blogEntryFile = this.BlogEntryFileService.GetById(fileId);
            this.BlogEntryFileService.Delete(blogEntryFile);

            if (this.Request.IsAjaxRequest())
            {
                var blogEntry = this.BlogEntryService.GetById(id);
                return PartialView(MVC.BlogEntryEdit.Views.BlogEntryFileControl, blogEntry.Files);
            }
            else
            {
                return this.RedirectToAction(MVC.BlogEntryEdit.Index(id));
            }
        }

        /// <summary>
        /// Validates a post.
        /// </summary>
        /// <param name="header">The header.</param>
        /// <param name="content">The content.</param>
        /// <returns>True if post is valid.</returns>
        private bool ValidatePost(string header, string content)
        {
            if (string.IsNullOrEmpty(header))
            {
                ModelState.AddModelError("header", "*");
            }

            if (string.IsNullOrEmpty(content))
            {
                ModelState.AddModelError("content", "*");
            }

            return ModelState.IsValid;
        }
    }
}